#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : deepinfra
# @Time         : 2024/10/31 14:04
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.schemas.image_types import ImageRequest, ImagesResponse
from meutils.notice.feishu import IMAGES, send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.apis.siliconflow.images import generate as siliconflow_generate

BASE_URL = "https://api.deepinfra.com"

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=6lr4oi"

DEFAULT_MODEL = "black-forest-labs/FLUX-1.1-pro"
MODELS = {

    "flux.1-schnell": "black-forest-labs/FLUX.1-schnell",
    "flux.1-dev": "black-forest-labs/FLUX.1-dev",
    "flux.1-pro": "black-forest-labs/FLUX.1-dev",
    "flux.1-pro-max": "black-forest-labs/FLUX.1-dev",

    "flux-schnell": "black-forest-labs/FLUX.1-schnell",
    "flux-dev": "black-forest-labs/FLUX.1-dev",
    "flux-pro": "black-forest-labs/FLUX.1-dev",
    "flux-pro-max": "black-forest-labs/FLUX.1-dev",

    "stable-diffusion-xl-base-1.0": "stabilityai/stable-diffusion-xl-base-1.0",  # 图生图
    "stable-diffusion-2-1": "stabilityai/stable-diffusion-2-1",  # 图生图

    "stable-diffusion": "stabilityai/stable-diffusion-3-medium",
    "stable-diffusion-3-medium": "stabilityai/stable-diffusion-3-medium",
    "stable-diffusion-3": "stabilityai/stable-diffusion-3-medium",

    "stable-diffusion-3-5-large": "stabilityai/stable-diffusion-3-5-large"

}

send_message = partial(
    _send_message,
    title=__name__,
    url=IMAGES
)


async def generate(request: ImageRequest, token: Optional[str] = None):
    try:
        token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL)
        headers = {"Authorization": f"Bearer {token}"}

        request.model = DEFAULT_MODEL  # MODELS.get(request.model, DEFAULT_MODEL)

        # Height of the generated image in pixels. Must be a multiple of 32 (Default: 1024, 256 ≤ height ≤ 1440)
        w, h = map(int, request.size.split("x"))

        max_size = max(w, h)
        w, h = w * 1440 / max_size, h * 1440 / max_size
        w, h = max(256, w), max(256, h)
        request.size = f"{int(w)}x{int(h)}"

        logger.debug(request)

        payload = {
            "prompt": request.prompt,
            "prompt_upsampling": True,
            "seed": request.seed,
            "safety_tolerance": 6,
            "width": w,
            "height": h
        }

        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
            response = await client.post(f"/v1/inference/{request.model}", json=payload)
            response.raise_for_status()
            data = response.json()

            return ImagesResponse(image=data["image_url"], metadata=data)

    except Exception as e:
        logger.error(e)
        send_message(f"生成失败：{e}, 使用siliconflow重试")
        return await siliconflow_generate(request)


if __name__ == '__main__':
    arun(generate(ImageRequest(prompt="一只可爱的猫", size="1440x1440")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2024/12/20 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import httpx

from meutils.pipe import *
from openai import AsyncClient
from meutils.caches import rcache
from openai._legacy_response import HttpxBinaryResponseContent


async def make_request_httpx(
        base_url: str,
        headers: Optional[dict] = None,

        path: Optional[str] = None,

        params: Optional[dict] = None,
        payload: Optional[dict] = None,
        data: Optional[Any] = None,
        files: Optional[dict] = None,
        timeout: Optional[int] = None,

        method: Optional[str] = None,

        debug: bool = False,
        **kwargs
):
    if method is None:
        method = (payload or data or files) and "POST" or "GET"

    path = path or "/"
    path = f"""/{path.removeprefix("/")}"""

    if debug:
        log = {
            "base_url": base_url,
            "path": path,
            "method": method,
            "headers": headers,
            "params": params,
            "payload": payload,
            "data": data,
            "timeout": timeout,
        }
        logger.debug(f"MAKE_REQUEST: {method.upper()} => {base_url}{path}")
        logger.debug(f"MAKE_REQUEST_DETAIL: {bjson(log)}")

    async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=timeout or 100) as client:
        # content: RequestContent | None = None,
        # data: RequestData | None = None,
        # files: RequestFiles | None = None,
        # json: typing.Any | None = None,
        # params: QueryParamTypes | None = None,
        # headers: HeaderTypes | None = None,
        response = await client.request(method, path, json=payload, data=data, files=files, params=params)
        # response.raise_for_status()

        if isinstance(response.content, HttpxBinaryResponseContent):
            return response.content

        try:
            return response.json()
        except Exception as e:
            logger.error(e)
            return response


async def make_request(
        base_url: str,
        api_key: Optional[str] = None,  # false 不走 Authorization bearer
        headers: Optional[dict] = None,

        path: Optional[str] = None,

        params: Optional[dict] = None,
        payload: Optional[dict] = None,
        files: Optional[dict] = None,

        method: Optional[str] = "POST",  # todo

        timeout: Optional[int] = None,

        debug: bool = False
):
    headers = headers or {}

    if headers:
        headers = {k: v for k, v in headers.items() if '_' not in k}
        if not any(i in base_url for i in {"queue.fal.run", "elevenlabs"}):  # todo  xi-api-key
            headers = {}

    client = AsyncClient(base_url=base_url, api_key=api_key, default_headers=headers, timeout=timeout)

    if not method:
        method = (payload or files) and "POST" or "GET"

    options = {}
    if params:
        options["params"] = params

    path = path or "/"
    path = f"""/{path.removeprefix("/")}"""

    logger.debug(f"MAKE_REQUEST: {method.upper()} => {base_url}{path}")
    if debug:
        log = {
            "base_url": base_url,
            "path": path,
            "method": method,
            "headers": headers,
            "params": params,
            "payload": payload,
            "files": files,
            "api_key": api_key,
            "timeout": timeout,
            "options": options,
        }
        logger.debug(f"MAKE_REQUEST_DETAIL: {bjson(log)}")

    if method.upper() == 'GET':
        try:
            response = await client.get(path, options=options, cast_to=object)
            return response
        except Exception as e:
            logger.error(e)

            headers = {
                "Authorization": f"Bearer {api_key}",
                **headers
            }

            async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=timeout or 100) as client:
                response = await client.get(path, params=params)

                if not any(i in base_url for i in {"queue.fal.run", "ppinfra", "ppio"}):  # 某些网站不正确返回
                    response.raise_for_status()

                # logger.debug(response.text)

                return response.json()

    elif method.upper() == 'POST':
        # if any("key" in i.lower() for i in headers or {}):  # 跳过Bearer鉴权
        #     async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=timeout or 100) as client:
        #         response = await client.post(path, json=payload, params=params)
        #         # response.raise_for_status()
        #
        #         # print(response.text)
        #
        #         return response.json()

        response = await client.post(path, body=payload, options=options, files=files, cast_to=object)

        # HttpxBinaryResponseContent

        return response


@rcache(ttl=1 * 24 * 3600)  # todo: 可调节
async def make_request_with_cache(
        base_url: str,
        api_key: Optional[str] = None,
        headers: Optional[dict] = None,

        path: Optional[str] = None,

        params: Optional[dict] = None,
        payload: Optional[dict] = None,
        files: Optional[dict] = None,

        method: str = "POST",

        timeout: Optional[int] = None,
        ttl=1 * 24 * 3600
):
    return await make_request(
        base_url=base_url,
        api_key=api_key,
        headers=headers,
        path=path,
        params=params,
        payload=payload,
        files=files,
        method=method,
        timeout=timeout,
    )


if __name__ == '__main__':
    from meutils.io.files_utils import to_bytes

    base_url = "https://api.chatfire.cn/tasks/kling-57751135"
    base_url = "https://httpbin.org"

    # arun(make_request(base_url=base_url, path='/ip'))

    base_url = "https://ai.gitee.com/v1"
    path = "/images/mattings"
    headers = {
        "Authorization": "Bearer WPCSA3ZYD8KBQQ2ZKTAPVUA059J2Q47TLWGB2ZMQ",
        "X-Package": "1910"
    }
    payload = {
        "model": "RMBG-2.0",
        "image": "path/to/image.png"
    }
    files = {
        "image": ('path/to/image.png', to_bytes("https://oss.ffire.cc/files/kling_watermark.png"))
    }
    #
    # arun(make_request(base_url=base_url,
    #                   path=path,
    #                   method="post",
    #                   files=files,
    #                   payload=payload,
    #
    #                   api_key="WPCSA3ZYD8KBQQ2ZKTAPVUA059J2Q47TLWGB2ZMQ"))

    base_url = "https://queue.fal.run/fal-ai"
    path = "vidu/q1/reference-to-video"
    payload = {
        "prompt": "A young woman and a monkey inside a colorful house",
        "reference_image_urls": [
            "https://v3.fal.media/files/panda/HDpZj0eLjWwCpjA5__0l1_0e6cd0b9eb7a4a968c0019a4eee15e46.png",
            "https://v3.fal.media/files/zebra/153izt1cBlMU-TwD0_B7Q_ea34618f5d974653a16a755aa61e488a.png",
            "https://v3.fal.media/files/koala/RCSZ7VEEKGFDfMoGHCwzo_f626718793e94769b1ad36d5891864a4.png"
        ],
        "aspect_ratio": "16:9",
        "movement_amplitude": "auto"
    }
    FAL_KEY = "aa5c047f-2621-4be2-9cee-9857a630aa11:b06782c97dffb50bfd6eebb63f49c624"

    headers ={"Authorization": f"key {FAL_KEY}"}
    arun(make_request(
        base_url=base_url,
        path=path,
        api_key=FAL_KEY,
        payload=payload,
        headers=headers,
        method="post",
        debug=True
    ))

    # fal - topaz - upscale - video
    FAL_KEY = "aa5c047f-2621-4be2-9cee-9857a630aa11:b06782c97dffb50bfd6eebb63f49c624"
    REQUEST_ID = "714e4d31-d735-45f7-a9f5-e50eecbb0743"
    base_url = "https://queue.fal.run/fal-ai"
    path = f"/minimax/requests/{REQUEST_ID}"
    # path=f"/kling-video/requests/{REQUEST_ID}/status"
    # "MAKE_REQUEST: GET => https://queue.fal.run/fal-ai/kling-video/requests/f570c7b0-b0f2-444b-b8c1-0212168f2f2e"
    headers = {
        "Authorization": f"key {FAL_KEY}"
    }
    # arun(make_request(
    #     base_url=base_url,
    #     path=path,
    #     headers=headers,
    #     method="get",
    #     debug=True
    # ))

    # 'detail': 'Request is still in progress',

    # base_url = "https://open.bigmodel.cn/api/paas/v4/web_search"
    # payload = {
    #     "search_query": "周杰伦",
    #     "search_engine": "search_std",
    #     "search_intent": True
    # }
    # api_key = "e130b903ab684d4fad0d35e411162e99.PqyXq4QBjfTdhyCh"
    # headers ={
    #     "host":'xx'
    # }

    # r = arun(make_request(base_url, api_key=api_key, payload=payload, headers=headers))

    """
    要调用的搜索引擎编码。目前支持：
    search_std : 智谱基础版搜索引擎
    search_pro : 智谱高阶版搜索引擎，老用户查看原有调用方式
    search_pro_sogou :搜狗
    search_pro_quark : 夸克搜索
    search_pro_jina : jina.ai搜索
    search_pro_bing : 必应搜索
    """
    # search_std,search_pro,search_pro_sogou,search_pro_quark,search_pro_jina,search_pro_bing

    #
    # UPSTREAM_BASE_URL = "https://ai.gitee.com/v1"
    # UPSTREAM_API_KEY = "5PJFN89RSDN8CCR7CRGMKAOWTPTZO6PN4XVZV2FQ"
    # payload = {
    #     "input": [{"type": "text", "text": "...text to classify goes here..."}],
    #     "model": "Security-semantic-filtering"
    # }
    #
    # arun(make_request(
    #     base_url=UPSTREAM_BASE_URL,
    #     api_key=UPSTREAM_API_KEY,
    #     path="moderations",
    #     payload=payload,
    #     debug=True
    # ))

    # UPSTREAM_BASE_URL = "https://ark.cn-beijing.volces.com/api/v3"
    # path = "/contents/generations/tasks"
    # UPSTREAM_API_KEY = "8a907822-58ed-4e2f-af25-b7b358e3164c"
    # payload = {
    #     "model": "doubao-seedance-1-0-pro-250528",
    #     "content": [
    #         {
    #             "type": "text",
    #             "text": "多个镜头。一名侦探进入一间光线昏暗的房间。他检查桌上的线索，手里拿起桌上的某个物品。镜头转向他正在思索。 --ratio 16:9"
    #         }
    #     ]
    # }
    # arun(make_request(
    #     base_url=UPSTREAM_BASE_URL,
    #     api_key=UPSTREAM_API_KEY,
    #     path=path,
    #     payload=payload,
    #     debug=True
    # ))

    # UPSTREAM_BASE_URL = "https://open.bigmodel.cn/api/paas/v4"
    # UPSTREAM_API_KEY = "88b82799f3234a5aad130b0f74c7eb85.tBMTRh0h1IqbvMaw"
    # path="/videos/generations"
    # # API_KEY=sk-R6y5di2fR3OAxEH3idNZIc4sm3CWIS4LAzRfhxSVbhXrrIej
    # payload = {
    #     "model": "cogvideox-flash",
    #     "prompt": "比得兔开小汽车，游走在马路上，脸上的表情充满开心喜悦。",
    #     "duration": 10
    # }
    #
    # arun(make_request(
    #     base_url=UPSTREAM_BASE_URL,
    #     api_key=UPSTREAM_API_KEY,
    #     path=path,
    #     payload=payload,
    #     debug=True
    # ))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openai_images
# @Time         : 2024/10/16 08:54
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from openai import AsyncOpenAI

from meutils.pipe import *
from meutils.io.files_utils import to_base64, to_url
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.llm.openai_utils import to_openai_images_params
from meutils.llm.check_utils import check_token_for_siliconflow
from meutils.notice.feishu import IMAGES, send_message as _send_message
from meutils.decorators.retry import retrying

from meutils.apis.translator import deeplx
from meutils.apis.proxy.kdlapi import get_one_proxy

from meutils.schemas.translator_types import DeeplxRequest
from meutils.schemas.image_types import ImageRequest, FluxImageRequest, SDImageRequest, ImagesResponse

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=3aA5dH"
FEISHU_URL_FREE = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=xlvlrH"

BASE_URL = "https://api.siliconflow.cn/v1"

DEFAULT_MODEL = "black-forest-labs/FLUX.1-schnell"
MODELS = {

    "flux.1-schnell": "black-forest-labs/FLUX.1-schnell",
    "flux.1-dev": "black-forest-labs/FLUX.1-dev",
    "flux.1-pro": "black-forest-labs/FLUX.1-dev",
    "flux.1.1-pro": "black-forest-labs/FLUX.1-dev",
    "flux-1.1-pro": "black-forest-labs/FLUX.1-dev",  # replicate
    "flux1.1-pro": "black-forest-labs/FLUX.1-dev",

    "flux.1-pro-max": "black-forest-labs/FLUX.1-dev",

    "flux-schnell": "black-forest-labs/FLUX.1-schnell",
    "flux-dev": "black-forest-labs/FLUX.1-dev",
    "flux-pro": "black-forest-labs/FLUX.1-dev",
    "flux-pro-max": "black-forest-labs/FLUX.1-dev",

    "stable-diffusion-xl-base-1.0": "stabilityai/stable-diffusion-xl-base-1.0",  # 图生图
    "stable-diffusion-2-1": "stabilityai/stable-diffusion-2-1",  # 图生图

    "stable-diffusion": "stabilityai/stable-diffusion-3-medium",
    "stable-diffusion-3-medium": "stabilityai/stable-diffusion-3-medium",
    "stable-diffusion-3": "stabilityai/stable-diffusion-3-medium",

    "stable-diffusion-3-5-large": "stabilityai/stable-diffusion-3-5-large",

    "stabilityai": "stabilityai/stable-diffusion-3-5-large",

}

send_message = partial(
    _send_message,
    title=__name__,
    url=IMAGES
)
check_token = check_token_for_siliconflow

check_valid_token = partial(check_token_for_siliconflow, threshold=-1)


@retrying(max_retries=3, title=__name__)
async def generate(request: ImageRequest, api_key: Optional[str] = None):
    request.prompt_enhancement = True
    if not request.prompt:
        # {'model': 'flux-schnell', 'messages': [{'role': 'user', 'content': '写一个10个字的冷笑话'}]}
        return ImagesResponse(**request.model_dump())

    if not request.model.startswith(("flux", "black-forest-labs")):  # 自动翻译
        request.prompt = deeplx.llm_translate(request.prompt)

    request.model = MODELS.get(request.model, DEFAULT_MODEL)
    # logger.debug(request)

    if any(i in request.model.lower() for i in {"pro-max", "pro"}):
        request.num_inference_steps = 20
        api_key = api_key or await get_next_token_for_polling(
            FEISHU_URL,
            check_token=check_token,
            from_redis=True,
            min_points=0.1
        )

    elif any(i in request.model.lower() for i in {"dev", "pro"}):  # 压缩像素
        request.num_inference_steps = 20
        api_key = api_key or await get_next_token_for_polling(
            FEISHU_URL,
            check_token=check_token,
            from_redis=True,
            min_points=0.1
        )
        # request.size = "2048x1024"

        w, h = map(int, request.size.split("x"))
        max_size = max(w, h)
        w, h = w * 1024 / max_size, h * 1024 / max_size
        request.size = f"{int(w)}x{int(h)}"
    else:
        api_key = api_key or await get_next_token_for_polling(FEISHU_URL_FREE, check_valid_token, from_redis=True)

    data = to_openai_images_params(request)
    # logger.debug(data)

    # 代理
    http_client = None
    if request.user == 'proxy':
        http_client = httpx.AsyncClient(proxy=await get_one_proxy(), timeout=60)

    client = AsyncOpenAI(base_url=BASE_URL, api_key=api_key, http_client=http_client, timeout=60)
    response = await client.images.generate(**data)
    response.model = ""
    # logger.debug(response)

    # response.data[0].url = response.data[0].url.replace(r'\u0026', '&')
    send_message(f"request: {request.model}\n{request.prompt}\nresponse: {response.data[0].url}",)

    response.data[0].url = await to_url(response.data[0].url, content_type="image/png")
    if request.response_format == "b64_json":
        b64_json = await to_base64(response.data[0].url)

        response.data[0].url = None
        response.data[0].b64_json = b64_json
    if response.data[0].url:
        return response
    else:
        raise ValueError("no image")


if __name__ == '__main__':
    from meutils.pipe import *

    data = {
        # "model": "flux-schnell",
        "model": "stable-diffusion-3-5-large",

        "prompt": "(Chinese dragon soaring through the clouds).(majestic, colorful, mythical, powerful, ancient).(DSLR camera).(wide-angle lens).(dawn)(fantasy photography).(Kodak Ektar 100)",
        "negative_prompt": "",
        "n": 1,
        # "response_format": "url",
        # "response_format": "b64_json",

        # "size": "16x9",
        "num_inference_steps": 20,
        "seed": None
    }

    # data = {'model': 'flux1.1-pro',
    #         'prompt': 'Surrealism, Chinese art, fairy, close-up of the upper body, hazy, glowing, dreamy, light pink and light blue gradient long hair, beautiful and charming, dressed in cashmere-like clothing, streamlined design, elegant, fair-skinned and beautiful, delicate features, comfortable, lying on an ice blue bed, background of a crescent moon and starry sky, with a touch of romantic gifts, virtual engine rendering, 3D model, OC rendering, perfect composition, ultra-detailed details, 3D rendering close-up shot. (flux1.1-pro)',
    #         'negative_prompt': '', 'n': 1, 'response_format': 'url', 'size': '1152x2048', 'num_inference_steps': 20,
    #         'seed': None}
    #
    # data = {
    #     "model": "flux-dev",
    #     "prompt": "(Chinese dragon soaring through the clouds).(majestic, colorful, mythical, powerful, ancient).(DSLR camera).(wide-angle lens).(dawn)(fantasy photography).(Kodak Ektar 100)",
    #     "negative_prompt": "",
    #     "n": 1,
    #     "response_format": "url",
    #     "size":"1366x768",
    #     "num_inference_steps": 20,
    #     "seed": None
    # }

    # request = FluxImageRequest(model="flux", prompt="a dog", size="1024x1024", num_inference_steps=1)
    # request = FluxImageRequest(model="flux-pro", prompt="a dog", size="10x10", num_inference_steps=1)

    data = {'model': 'flux-schnell', 'prompt': '画一个2025年电脑如何一键重装系统win10教程详解的封面图', 'n': 1,
            'size': '680x400'}
    request = FluxImageRequest(**data)

    print(request)
    # request = SDImageRequest(
    #     # model="stable-diffusion-2-1",
    #     # model="stable-diffusion-xl-base-1.0",
    #     model="stable-diffusion-3-5-large",
    #     # model="stable-diffusion",
    #
    #     prompt="an island near sea, with seagulls, moon shining over the sea, light house, boats int he background, fish flying over the sea",
    #     size="576x1024",
    # )

    arun(generate(request))

    # https://api.siliconflow.cn/v1



# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : __init__.py
# @Time         : 2020/11/26 2:57 下午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


class DB(object):

    def __init__(self, is_test=None):

        if is_test is None:
            if HOST_NAME.__contains__('local') or LOCAL:
                is_test = True

        self.is_test = is_test

    def redis(self, ips=None, password=None, batch=False):  # redis集群
        """
        # todo: 批量插入（[(k, v),]）or yield
        @param ips:
        @param password:
        @param batch:
        @return:
        """
        if self.is_test:
            return {}

        from rediscluster import RedisCluster  # pip install redis-py-cluster

        startup_nodes = [dict(zip(['host', 'port'], ip.split(':'))) for ip in ips]

        rc = RedisCluster(
            startup_nodes=startup_nodes,
            decode_responses=True,
            skip_full_coverage_check=True,
            password=password,
        )
        if batch:
            return rc.pipeline()  # DB().redis.execute()

        else:
            return rc


class Redis(object):
    """
    rc.delete
    rc.lpush('lpush', 'a', 'b') # ['b', 'a']
    rc.rpush('rpush', 'a', 'b') # ['a', 'b']
    rc.rpop

    rc.lrange('rpush', 0, 10)
    rc.lrange('lpush', 0, 10)

    rc.hset(name, key=None, value=None, mapping=None)
    rc.hget(name, key)
    rc.hmget(name, keys, *args)

    rc.sadd('sadd', 0)
    rc.smembers('sadd')

    rc.zadd('zadd', mapping={'a': 888, 'b': 1})
    rc.zrange('zadd', 0, 1000, desc=True)

    队列 https://www.cnblogs.com/arkenstone/p/7813551.html
    https://blog.csdn.net/youyou1543724847/article/details/86756625
    """

    def __init__(self, ips=None, password=None, decode_responses=False, ttl=3):

        if isinstance(ips, str):
            import redis

            self.rc = redis.Redis(
                *ips.split(':'),
                password=password,
                decode_responses=decode_responses
            )

        else:
            from rediscluster import RedisCluster

            startup_nodes = [dict(zip(['host', 'port'], ip.split(':'))) for ip in ips]

            self.rc = RedisCluster(
                startup_nodes=startup_nodes,
                password=password,
                decode_responses=decode_responses,  # get对象设置False decode
                skip_full_coverage_check=True,
            )

        self.rc.set_obj = self._set_obj
        self.rc.get_obj = self._get_obj

        # ttl
        cache_fn = ttl_cache(ttl, key=str)
        self.rc.get_ttl = cache_fn(self.rc.get)
        self.rc.hget_ttl = cache_fn(self.rc.hget)  # rc.hget(name, key)
        self.rc.get_obj_ttl = cache_fn(self._get_obj)

    @timer("RedisInsertData")
    def insert(self, values, insert_fn=lambda k, v, p: p.set(k, v)):
        with self.rc.pipeline(transaction=False) as p:  # 事务：原子性、一致性、隔离性、持久性

            _insert_fn = functools.partial(insert_fn, p=p)
            for v in tqdm(values):  # xJobs
                _insert_fn(*v)

            p.execute()

    def _set_obj(self, key, obj, ex=None, px=None, nx=False, xx=False, keepttl=False):
        self.rc.set(key, pickle.dumps(obj), ex, px, nx, xx, keepttl)  # dill/pickle

    def _get_obj(self, key, default=None):
        try:
            _ = self.rc.get(key)
            return pickle.loads(_) if _ else default

        except Exception as e:
            logger.error(e)


class SQL(object):

    def __init__(self, **kwargs):
        self.conn = None  # 重写

    def get_dataframe(self, sql, chunksize=None, **kwargs):
        return pd.read_sql(sql, self.conn, chunksize=chunksize, **kwargs)

    def set_dataframe(self, df: pd.DataFrame, table: str, if_exists='replace', index=False, chunksize=None, **kwargs):
        df.to_sql(table, self.conn, if_exists=if_exists, index=index, chunksize=chunksize, **kwargs)


class Sqlite(SQL):

    def __init__(self, db='test.db', **kwargs):
        super().__init__(**kwargs)

        import sqlite3
        self.conn = sqlite3.connect(db)
        self.sqlite_master = pd.read_sql('SELECT * from SQLITE_MASTER', self.conn)
        self.sqlite_version = pd.read_sql('SELECT SQLITE_VERSION()', self.conn)


@singleton
class MySQL(SQL):
    def __init__(self, url=None, echo=False, db_kwargs=None, **kwargs):
        """

        :param url: f'mysql+pymysql://{user}:{password}@{host}:{port}/{database}'
        :param echo:
        :param kwargs:
        """
        super().__init__(**kwargs)

        import pymysql
        from sqlalchemy import create_engine

        url = url or "mysql+pymysql://root:root123456@localhost/test?charset=utf8mb4"
        self.conn = create_engine(url, echo=echo, **db_kwargs or {})  # 不兼容

        try:
            _ = self.tables
        except Exception as e:
            logger.error(e)

            _ = re.split('[:@/?]', url[16:])
            self.conn = pymysql.Connect(**dict(zip(['user', 'password', 'host', 'database'], _)))
            self.conn.execute = self.conn.cursor().execute

    def query(self, sql):
        return pd.read_sql(sql, self.conn)

    def query_table(self, table_name):
        return pd.read_sql_table(table_name, self.conn)

    def create_table_or_upsert(
            self, df: pd.DataFrame,
            table_name: str,
            if_exists="append",  # "append": 不存在则创建，存在则存入数据
            primary_key=''
    ):
        """Upsert 是 update 和 insert 的组合

        :param df:
        :param table_name:
        :param if_exists:
            "append": 不存在则创建，存在则存入数据,
            "replace"
            "fail"
        :param primary_key:
        :return:
        """
        _ = df.to_sql(table_name, self.conn, index=False, if_exists=if_exists, method='multi')  # Literal[if_exists]

        if primary_key and primary_key in df:  # 初始化一次即可
            self.alter_primary_key(table_name, primary_key)

        return _

    @property
    def tables(self):
        return pd.read_sql('show tables', self.conn)

    @property
    def databases(self):
        return pd.read_sql('show databases', self.conn)

    def get_primary_key_info(self, table_name):
        df = pd.read_sql(f"SHOW KEYS FROM {table_name} WHERE Key_name = 'PRIMARY';", self.conn)
        return df.to_dict('records')

    def alter_primary_key(self, table_name, primary_key):
        """
            ALTER TABLE {table_name} ADD PRIMARY KEY ({field1}, {field2}, ...);
            ALTER TABLE table_name ADD PRIMARY KEY (column_name(length)); 字符串要指定长度
        :param table_name:
        :param primary_key:
        :return:
        """
        _ = self.get_primary_key_info(table_name)
        if _: logger.warning(f'exist primary_key: {_}')

        self.conn.execute("ALTER TABLE {table_name} DROP PRIMARY KEY")
        self.conn.execute(f"ALTER TABLE {table_name} ADD PRIMARY KEY ({primary_key})")

        rprint(self.get_primary_key_info(table_name))


def get_minio_client():
    from minio import Minio

    return Minio(
        endpoint=os.getenv('MINIO_ENDPOINT'),
        access_key=os.getenv('MINIO_ACCESS_KEY'),
        secret_key=os.getenv('MINIO_SECRET_KEY'),
        secure=False)


if __name__ == '__main__':
    # rc = Redis('9736:071.4.9.931'[::-1], password='b***e', decode_responses=True).rc
    #
    # rc.set('a', 'xxxx')
    #
    # print(rc.get('a'))

    db = MySQL()
    print(db.tables)



import json
import traceback
from functools import partial
from httpx import HTTPStatusError
from openai import APIStatusError

from fastapi import status
from starlette.requests import Request
from starlette.responses import JSONResponse

from fastapi.exceptions import RequestValidationError, HTTPException

from meutils.notice.feishu import send_message_for_http as send_message


async def http_error_handler(_: Request, exc: HTTPException) -> JSONResponse:
    # print(exc)
    content = {
        "error":
            {
                "message": f"{exc.detail}",
                "type": "http-error",
            }
    }
    return JSONResponse(
        content=content,
        status_code=exc.status_code
    )


async def general_exception_handler(request: Request, exc: Exception):
    return JSONResponse(
        content={"message": str(exc)},
        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
    )


async def chatfire_api_exception_handler(request: Request, exc: Exception):
    content = {
        "error":
            {
                "message": f"{exc}",
                "type": "cf-api-error",
            },

        "code": status.HTTP_500_INTERNAL_SERVER_ERROR
    }

    # 默认值
    reps = None
    if isinstance(exc, (HTTPStatusError, APIStatusError)):
        status_code = exc.response.status_code or 500

        content['code'] = status_code
        content['error']['message'] = f"{exc.response.text}"

        # 置换错误
        for i in {"fal.ai", "api.ppinfra"}:
            content['error']['message'] = content['error']['message'].replace(i, 'AI')

        reps = JSONResponse(
            content=content,
            status_code=status_code,
        )

    # send_message
    content_detail = f"{traceback.format_exc()}"  # 是不是最后几行就可以了
    #
    # from meutils.pipe import logger
    # logger.debug(content_detail)

    if any(code in content_detail for code in {'451', }):
        content_detail = ""

    send_message([content, content_detail], title=__name__)

    # from meutils.pipe import logger
    #
    # try:
    #
    #     logger.debug(request.headers)
    #     logger.debug(request.url)
    #     logger.debug(request.method)
    #     logger.debug(request.query_params._dict)
    #
    #     logger.debug(request.client)
    #     logger.debug(request.cookies)
    #
    #     payload  = await request.body()
    #
    #     # send_message(payload, title=__name__)
    #
    #     logger.debug(payload)
    #     logger.debug(payload)
    #
    #
    # except Exception as e:
    #     logger.debug(e)

    return reps or JSONResponse(
        content=content,
        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
    )


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : image
# @Time         : 2024/10/11 15:01
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json_repair

from meutils.pipe import *
from meutils.decorators.retry import retrying, IgnoredRetryException
from meutils.schemas.yuanbao_types import FEISHU_URL, YUANBAO_BASE_URL
from meutils.schemas.image_types import HunyuanImageProcessRequest

from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.io.files_utils import to_url_fal as to_url
from meutils.notice.feishu import send_message


@retrying(min=3, ignored_exception_types=(IgnoredRetryException,))
async def image_process(request: HunyuanImageProcessRequest, token: Optional[str] = None):
    token = token or await get_next_token_for_polling(FEISHU_URL)

    payload = {
        "imageUrl": request.image if request.image.startswith('http') else await to_url(request.image),
    }
    if request.task == 'style':
        payload.update({
            "style": request.style,
            "prompt": f"转换为{request.style}",
        })

    headers = {
        'cookie': token
    }
    async with httpx.AsyncClient(base_url=YUANBAO_BASE_URL, headers=headers, timeout=100) as client:
        response = await client.post(f"/api/image/{request.task}", json=payload)
        response.raise_for_status()
        logger.debug(response.text)

        # 新版本的位置可能不一样 注意
        # data = json_repair.repair_json(
        #     response.text.replace(r'\u0026', '&').rsplit("data: [TRACEID", 1)[0],
        #     return_objects=True
        # )[-1]

        skip_strings = ['DONE', 'TRACEID']
        data = response.text.replace(r'\u0026', '&').splitlines() | xsse_parser(skip_strings=skip_strings)
        data = data and data[-1]
        logger.debug(data)

        # todo: 错误处理
        if isinstance(data, dict) and any(data["code"] == code for code in {"429"}):
            Exception(f"重试: {response.text}")

        elif isinstance(data, list) or any(i in response.text for i in {"当前图片没有检测到水印"}):  # 跳过重试并返回原始错误
            raise IgnoredRetryException(f"忽略重试: \n{response.text}")

        data = {
            "data": [
                {
                    "url": data["imageUrl"],
                    "imageUrl": data["imageUrl"],
                    "thumbnailUrl": data["thumbnailUrl"],
                }
            ]
        }

        return data


if __name__ == '__main__':
    # request = ImageProcessRequest(image="https://oss.ffire.cc/files/kling_watermark.png", task='removewatermark')
    token = "web_uid=ac283ec7-4bf6-40c9-a0ce-5a2e0cd7db06; hy_source=web; hy_user=I09MgMfFcUUyVSIg; hy_token=hevVCi/QuVjQcre5NDRMO7FuiWCZoDMIq3Zp8IwNxrPUofl4zWYazHEdeZ2S5o7q; _qimei_q36=; _qimei_h38=f2d27f50f0f23e085296d28303000006a17a09; _qimei_fingerprint=efbb885a22f7d4e5589008c28bc8e7ba; _qimei_uuid42=18c0310102d1002a082420cd40bb9717523c3c7e12; _gcl_au=1.1.915258067.1733278380; _ga_RPMZTEBERQ=GS1.1.1733722091.3.1.1733722108.0.0.0; _ga=GA1.2.981511920.1725261466; sensorsdata2015jssdkcross=%7B%22distinct_id%22%3A%22100000458739%22%2C%22first_id%22%3A%22191b198c7b2d52-0fcca8d731cb9b8-18525637-2073600-191b198c7b31fd9%22%2C%22props%22%3A%7B%22%24latest_traffic_source_type%22%3A%22%E7%A4%BE%E4%BA%A4%E7%BD%91%E7%AB%99%E6%B5%81%E9%87%8F%22%2C%22%24latest_utm_medium%22%3A%22cpc%22%7D%2C%22identities%22%3A%22eyIkaWRlbnRpdHlfY29va2llX2lkIjoiMTkxYjE5OGM3YjJkNTItMGZjY2E4ZDczMWNiOWI4LTE4NTI1NjM3LTIwNzM2MDAtMTkxYjE5OGM3YjMxZmQ5IiwiJGlkZW50aXR5X2xvZ2luX2lkIjoiMTAwMDAwNDU4NzM5In0%3D%22%2C%22history_login_id%22%3A%7B%22name%22%3A%22%24identity_login_id%22%2C%22value%22%3A%22100000458739%22%7D%2C%22%24device_id%22%3A%22191b198c7b2d52-0fcca8d731cb9b8-18525637-2073600-191b198c7b31fd9%22%7D"
    with timer():
        image = "https://sfile.chatglm.cn/chatglm4/3dcb1cc2-22ad-420b-9d16-dc71dffc02b2.png"
        image = "https://oss.ffire.cc/files/kling_watermark.png"
        image = "https://cdn.meimeiqushuiyin.cn/2024-12-05/ori/tmp_0511a4cb2066ffc309fa6f7a733ac1e93236709bf46c9430.jpg"
        image = "https://cdn.meimeiqushuiyin.cn/2024-12-05/ori/tmp_de5e186878b079a87d22c561f17e6853.jpg"
        image = "https://yuanbao.tencent.com/api/resource/download?resourceId=740fed65baed5f763db891b5443b7ee5"
        image = url = "https://oss.ffire.cc/files/shuiyin.jpg"
        # image = url = "https://oss.ffire.cc/files/shuiyin2.jpg"

        request = HunyuanImageProcessRequest(image=image, task='removewatermark')

        arun(image_process(request, token=token))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/4/1 14:11
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
import base64
from openai import OpenAI
from PIL import Image
from io import BytesIO

client = OpenAI(
    api_key=os.getenv("GOOGLE_API_KEY"),
    base_url=os.getenv("GOOGLE_BASE_URL"),
)

response = client.images.generate(
    model="imagen-3.0-generate-002",
    prompt="a portrait of a sheepadoodle wearing a cape",
    response_format='b64_json',
    n=1,
)
#
# for image_data in response.data:
#     image = Image.open(BytesIO(base64.b64decode(image_data.b64_json)))
#     image.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : lip_sync
# @Time         : 2025/1/3 16:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
1. 上传图片 image_to_avatar检测
2. 上传视频 video_to_avatar检测
3. 上传音频+创建任务

"""
import asyncio

from meutils.pipe import *
from meutils.str_utils.json_utils import json_path

from meutils.schemas.jimeng_types import BASE_URL, MODELS_MAP, FEISHU_URL
from meutils.schemas.video_types import LipsyncVideoRequest
from meutils.schemas.task_types import TaskResponse
from meutils.apis.jimeng.common import get_headers, check_token
from meutils.apis.jimeng.files import upload_for_image, upload_for_video

from meutils.config_utils.lark_utils import get_next_token_for_polling

from fake_useragent import UserAgent

ua = UserAgent()


async def create_realman_avatar(image_url: str, token: str):
    if image_url.startswith("http"):
        image_url = await upload_for_image(image_url, token)

    url = "/mweb/v1/create_realman_avatar"
    headers = get_headers(url, token)

    payload = {
        "input_list": [
            {
                "image_uri": image_url,
                "submit_id": str(uuid.uuid4()),
                "mode": 0
            },
            {
                "image_uri": image_url,
                "submit_id": str(uuid.uuid4()),
                "mode": 1
            }
        ]
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(url, json=payload)
        response.raise_for_status()
        data = response.json()
        logger.debug(bjson(data))  # 1914628189186

        response = TaskResponse(metadata=data, system_fingerprint=token)
        if task_ids := json_path(data, "$..task_id"):  # 返回 imageurl vid
            response.task_id = ' '.join(task_ids)
            return response

        else:
            response.message = str(json_path(data, "$..message"))
            response.status = "fail"
            return response


async def get_task(task_id: str, token: str = "916fed81175f5186a2c05375699ea40d"):
    """
    $..image_to_avatar 成功： 先检测图片or视频
    :param task_ids:
    :return:
    """
    task_ids = task_id.split()

    url = "/mweb/v1/mget_generate_task"
    headers = get_headers(url, token)

    payload = {"task_id_list": task_ids}
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(url, json=payload)
        response.raise_for_status()
        data = response.json()
        logger.debug(bjson(data))

        if json_path(data, "$..image_to_avatar"):  # 角色检测 create_realman_avatar
            resource_id_std = resource_id_loopy = ""
            if resource_id_stds := json_path(data, "$..resource_id_std"):
                resource_id_std = "".join(resource_id_stds)

            if resource_id_loopys := json_path(data, "$..resource_id_loopy"):
                resource_id_loopy = "".join(resource_id_loopys)

            task_data = {
                "resource_id_std": resource_id_std,
                "resource_id_loopy": resource_id_loopy
            }
            response = TaskResponse(task_id=task_id, data=task_data, metadata=data)
            if resource_id_std and resource_id_loopy:
                response.status = "success"

            if (message := json_path(data, "$..image_to_avatar.message")) and "fail" in str(message).lower():
                response.message = str(message)
                response.status = "fail"

            return response

        else:
            response = TaskResponse(task_id=task_id, metadata=data)
            if (message := json_path(data, "$..fail_msg")) and "success" not in str(message).lower():
                response.message = str(message)
                response.status = "fail"
                return response

            if will_cost := json_path(data, "$..will_cost"):
                response.will_cost = will_cost[0]

            if video_urls := json_path(data, "$..[360p,480p,720p].video_url"):
                response.data = [{"video": _} for _ in video_urls]

            return response


async def create_task(request: LipsyncVideoRequest, token: Optional[str] = None):
    # token = token or await get_next_token_for_polling(FEISHU_URL, check_token)
    token = "7c5e148d9fa858e3180c42f843c20454"  # 年付
    token = "916fed81175f5186a2c05375699ea40d"  # 月付

    url = "/mweb/v1/batch_generate_video"

    headers = get_headers(url, token)

    model = request.model
    scene = "lip_sync_image"
    image_url = await upload_for_image(request.image_url, token)

    # 角色检测
    realman_avatar_response = await create_realman_avatar(image_url, token)
    if realman_avatar_response.status == "fail":
        return realman_avatar_response

    else:
        for _ in range(10):
            task_response = await get_task(realman_avatar_response.task_id, token)
            if task_response.status == "fail":
                logger.debug("fail")
                return task_response
            elif task_response.status == "success":
                logger.debug("success")

                realman_avatar_response = task_response
                break
            else:
                await asyncio.sleep(3)
                continue

    audio_vid, audio_url = await upload_for_video(request.audio_url, token)

    resource_id_std = realman_avatar_response.data.get("resource_id_std")
    resource_id_loopy = realman_avatar_response.data.get("resource_id_loopy")

    i2v_opt = v2v_opt = {}
    if request.video_url:
        v2v_opt = {}

    # payload = {
    #     "submit_id": "",
    #     "task_extra": "{\"promptSource\":\"photo_lip_sync\",\"generateTimes\":1,\"lipSyncInfo\":{\"sourceType\":\"local-file\",\"name\":\"vyFWygmZsIZlUO4s0nr2n.wav\"},\"isUseAiGenPrompt\":false,\"batchNumber\":1}",
    #     "http_common_info": {
    #         "aid": 513695
    #     },
    #     "input": {
    #         "seed": 3112889115,
    #         "video_gen_inputs": [
    #             {
    #                 "v2v_opt": {},
    #                 "i2v_opt": {
    #                     "realman_avatar": {
    #                         "enable": True,
    #                         "origin_image": {
    #                             # "width": 800,
    #                             # "height": 1200,
    #                             "image_uri": "tos-cn-i-tb4s082cfz/4dead1bfc8e84572a91f2e047016a351",
    #                             "image_url": ""
    #                         },
    #                         "origin_audio": {
    #                             # "duration": 9.976625,
    #                             "vid": "v02870g10004cu8d4r7og65j2vr5opb0"
    #                         },
    #
    #                         "resource_id_std": "381c534f-bcef-482e-8f17-5b30b64e41a1",
    #                         "resource_id_loopy": "b9ac51cb-e26c-4b63-81d9-34ed24053032",
    #                         #
    #                         # "tts_info": "{\"name\":\"vyFWygmZsIZlUO4s0nr2n.wav\",\"source_type\":\"local-file\"}"
    #                     }
    #                 },
    #                 "audio_vid": "v02870g10004cu8d4r7og65j2vr5opb0",
    #                 "video_mode": 4
    #             }
    #         ]
    #     },
    #     "mode": "workbench",
    #     "history_option": {},
    #     "commerce_info": {
    #         "resource_id": "generate_video",
    #         "resource_id_type": "str",
    #         "resource_sub_type": "aigc",
    #         "benefit_type": "lip_sync_avatar_std",  # 5积分
    #         # "benefit_type": "lip_sync_avatar_lively" # 10积分
    #     },
    #     "scene": "lip_sync_image",
    #     "client_trace_data": {},
    #     "submit_id_list": [
    #         str(uuid.uuid4())
    #     ]
    # }

    if request.image_url:
        i2v_opt = {
            "realman_avatar": {
                "enable": True,
                "origin_image": {
                    "image_uri": image_url,
                    "image_url": ""
                },
                "resource_id_loopy": resource_id_loopy,
                "resource_id_std": resource_id_std,
                "origin_audio": {
                    "vid": audio_vid
                },
                # "tts_info": "{\"name\":\"vyFWygmZsIZlUO4s0nr2n.wav\",\"source_type\":\"local-file\"}"
            }
        }

    payload = {
        "submit_id": "",
        # "task_extra": "{\"promptSource\":\"photo_lip_sync\",\"generateTimes\":1,\"lipSyncInfo\":{\"sourceType\":\"local-file\",\"name\":\"vyFWygmZsIZlUO4s0nr2n.wav\"},\"isUseAiGenPrompt\":false,\"batchNumber\":1}",
        "http_common_info": {
            "aid": 513695
        },
        "input": {
            "seed": 2032846910,
            "video_gen_inputs": [
                {
                    "v2v_opt": v2v_opt,
                    "i2v_opt": i2v_opt,
                    "audio_vid": audio_vid,
                    "video_mode": 4
                }
            ]
        },
        "mode": "workbench",
        "history_option": {},
        "commerce_info": {
            "resource_id": "generate_video",
            "resource_id_type": "str",
            "resource_sub_type": "aigc",
            "benefit_type": model,
            # "benefit_type": "lip_sync_avatar_lively" # 10积分
        },
        "scene": scene,
        "client_trace_data": {},
        "submit_id_list": [
            str(uuid.uuid4())
        ]
    }

    logger.debug(bjson(payload))

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(url, json=payload)
        response.raise_for_status()
        data = response.json()
        logger.debug(bjson(data))

    if task_ids := json_path(data, "$..task.task_id"):
        task_id = task_ids[0]
        return TaskResponse(task_id=task_id, system_fingerprint=token)


# {
#     "submit_id": "740e28e3-12fd-4ab6-82da-7f2028ac6314",
#     "task_extra": "{\"promptSource\":\"custom\",\"originSubmitId\":\"3575ebec-1d35-42f1-bd19-6cf0c8dee0b1\",\"isDefaultSeed\":1,\"originTemplateId\":\"\",\"imageNameMapping\":{},\"isUseAiGenPrompt\":false,\"batchNumber\":1}",
#     "input": {
#         "video_aspect_ratio": "16:9",
#         "seed": 840565633,
#         "video_gen_inputs": [
#             {
#                 "prompt": "现代几何构图海报模板，庆祝男演员都市爱情剧《街角晚风》成功。画面分割为几个深红和灰色块面。一个灰色块内展示清晰的德塔文景气指数上升曲线图（深红线条）。一个色块放置男演员侧脸剧照。剧名和标语“人气飙升，全城热恋”分布在不同色块上，字体设计现代。整体风格简约、结构化、高级。",
#                 "fps": 24,
#                 "duration_ms": 5000,
#                 "video_mode": 2,
#                 "template_id": ""
#             }
#         ],
#         "priority": 0,
#         "model_req_key": "dreamina_ic_generate_video_model_vgfm_3.0"
#     },
#     "mode": "workbench",
#     "history_option": {},
#     "commerce_info": {
#         "resource_id": "generate_video",
#         "resource_id_type": "str",
#         "resource_sub_type": "aigc",
#         "benefit_type": "basic_video_operation_vgfm_v_three"
#     },
#     "client_trace_data": {}
# # }

if __name__ == '__main__':
    token = "916fed81175f5186a2c05375699ea40d"

    request = LipsyncVideoRequest(
        model="lip_sync_avatar_std",
        image_url="https://oss.ffire.cc/files/kling_watermark.png",
        video_url="",
        audio_url="https://oss.ffire.cc/files/lipsync.mp3"
    )

    # with timer():
    #     r = arun(create_realman_avatar(request.image_url, token))
    #     arun(get_task(r.task_id))

    # image_uri = "tos-cn-i-tb4s082cfz/387649a361e546f89549bd3510ab926d"
    # task_ids = arun(create_realman_avatar(image_uri, token="7c5e148d9fa858e3180c42f843c20454"))
    # arun(mget_generate_task(task_ids))
    with timer():
        r = arun(create_task(request))
    # arun(get_task(r.task_id))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : besttable
# @Time         : 2021/1/28 8:01 下午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : https://github.com/foutaise/texttable/blob/master/texttable.py

# TODO: 设置df输出精度
"""module for creating simple ASCII tables
Example:
    table = Besttable()
    table.set_cols_align(["l", "r", "c"])
    table.set_cols_valign(["t", "m", "b"])
    table.add_rows([["Name", "Age", "Nickname"],
                    ["Mr\\nXavier\\nHuon", 32, "Xav'"],
                    ["Mr\\nBaptiste\\nClement", 1, "Baby"],
                    ["Mme\\nLouise\\nBourgeau", 28, "Lou\\n\\nLoue"]])
    print table.draw() + "\\n"
    table = Besttable()
    table.set_deco(Besttable.HEADER)
    table.set_cols_dtype(['t',  # text
                          'f',  # float (decimal)
                          'e',  # float (exponent)
                          'i',  # integer
                          'a']) # automatic
    table.set_cols_align(["l", "r", "r", "r", "l"])
    table.add_rows([["text",    "float", "exp", "int", "auto"],
                    ["abcd",    "67",    654,   89,    128.001],
                    ["efghijk", 67.5434, .654,  89.6,  12800000000000000000000.00023],
                    ["lmn",     5e-78,   5e-78, 89.4,  .000000000000128],
                    ["opqrstu", .023,    5e+78, 92.,   12800000000000000000000]])
    print table.draw()
Result:
    +----------+-----+----------+
    |   Name   | Age | Nickname |
    +==========+=====+==========+
    | Mr       |     |          |
    | Xavier   |  32 |          |
    | Huon     |     |   Xav'   |
    +----------+-----+----------+
    | Mr       |     |          |
    | Baptiste |   1 |          |
    | Clement  |     |   Baby   |
    +----------+-----+----------+
    | Mme      |     |   Lou    |
    | Louise   |  28 |          |
    | Bourgeau |     |   Loue   |
    +----------+-----+----------+
    text   float       exp      int     auto
    ===========================================
    abcd   67.000   6.540e+02   89    128.001
    efgh   67.543   6.540e-01   90    1.280e+22
    ijkl   0.000    5.000e-78   89    0.000
    mnop   0.023    5.000e+78   92    1.280e+22
"""

from __future__ import division

__all__ = ["Besttable", "ArraySizeError"]

__author__ = 'Gerome Fournier <jef(at)foutaise.org>'
__license__ = 'MIT'
__version__ = '1.6.3'
__credits__ = """\
Jeff Kowalczyk:
    - textwrap improved import
    - comment concerning header output
Anonymous:
    - add_rows method, for adding rows in one go
Sergey Simonenko:
    - redefined len() function to deal with non-ASCII characters
Roger Lew:
    - columns datatype specifications
Brian Peterson:
    - better handling of unicode errors
Frank Sachsenheim:
    - add Python 2/3-compatibility
Maximilian Hils:
    - fix minor bug for Python 3 compatibility
frinkelpi:
    - preserve empty lines
"""

import sys
import unicodedata
from functools import reduce

unicode_type = str
bytes_type = bytes

# define a text wrapping function to wrap some text
# to a specific width:
# - use cjkwrap if available (better CJK support)
# - fallback to textwrap otherwise
try:
    import cjkwrap


    def textwrapper(txt, width):
        return cjkwrap.wrap(txt, width)
except ImportError:
    try:
        import textwrap


        def textwrapper(txt, width):
            return textwrap.wrap(txt, width)
    except ImportError:
        sys.stderr.write("Can't import textwrap module!\n")
        raise

# define a function to calculate the rendering width of a unicode character
# - use wcwidth if available
# - fallback to unicodedata information otherwise
try:
    import wcwidth


    def uchar_width(c):
        """Return the rendering width of a unicode character
        """
        return max(0, wcwidth.wcwidth(c))
except ImportError:
    def uchar_width(c):
        """Return the rendering width of a unicode character
        """
        if unicodedata.east_asian_width(c) in 'WF':
            return 2
        elif unicodedata.combining(c):
            return 0
        else:
            return 1


def obj2unicode(obj):
    """Return a unicode representation of a python object
    """
    if isinstance(obj, unicode_type):
        return obj
    elif isinstance(obj, bytes_type):
        try:
            return unicode_type(obj, 'utf-8')
        except UnicodeDecodeError as strerror:
            sys.stderr.write("UnicodeDecodeError exception for string '%s': %s\n" % (obj, strerror))
            return unicode_type(obj, 'utf-8', 'replace')
    else:
        return unicode_type(obj)


def len(iterable):
    """Redefining len here so it will be able to work with non-ASCII characters
    """
    if isinstance(iterable, bytes_type) or isinstance(iterable, unicode_type):
        return sum([uchar_width(c) for c in obj2unicode(iterable)])
    else:
        return iterable.__len__()


class ArraySizeError(Exception):
    """Exception raised when specified rows don't fit the required size
    """

    def __init__(self, msg):
        self.msg = msg
        Exception.__init__(self, msg, '')

    def __str__(self):
        return self.msg


class FallbackToText(Exception):
    """Used for failed conversion to float"""
    pass


class Besttable:
    BORDER = 1
    HEADER = 1 << 1
    HLINES = 1 << 2
    VLINES = 1 << 3

    def __init__(self, max_width=80):
        """Constructor
        - max_width is an integer, specifying the maximum width of the table
        - if set to 0, size is unlimited, therefore cells won't be wrapped
        """

        self.set_max_width(max_width)
        self._precision = 3

        self._deco = Besttable.VLINES | Besttable.HLINES | Besttable.BORDER | \
                     Besttable.HEADER
        self.set_chars(['-', '|', '+', '='])
        self.reset()

    def reset(self):
        """Reset the instance
        - reset rows and header
        """

        self._hline_string = None
        self._row_size = None
        self._header = []
        self._rows = []
        return self

    def set_max_width(self, max_width):
        """Set the maximum width of the table
        - max_width is an integer, specifying the maximum width of the table
        - if set to 0, size is unlimited, therefore cells won't be wrapped
        """
        self._max_width = max_width if max_width > 0 else False
        return self

    def set_chars(self, array):
        """Set the characters used to draw lines between rows and columns
        - the array should contain 4 fields:
            [horizontal, vertical, corner, header]
        - default is set to:
            ['-', '|', '+', '=']
        """

        if len(array) != 4:
            raise ArraySizeError("array should contain 4 characters")
        array = [x[:1] for x in [str(s) for s in array]]
        (self._char_horiz, self._char_vert,
         self._char_corner, self._char_header) = array
        return self

    def set_deco(self, deco):
        """Set the table decoration
        - 'deco' can be a combinaison of:
            Besttable.BORDER: Border around the table
            Besttable.HEADER: Horizontal line below the header
            Besttable.HLINES: Horizontal lines between rows
            Besttable.VLINES: Vertical lines between columns
           All of them are enabled by default
        - example:
            Besttable.BORDER | Besttable.HEADER
        """

        self._deco = deco
        return self

    def set_header_align(self, array):
        """Set the desired header alignment
        - the elements of the array should be either "l", "c" or "r":
            * "l": column flushed left
            * "c": column centered
            * "r": column flushed right
        """

        self._check_row_size(array)
        self._header_align = array
        return self

    def set_cols_align(self, array):
        """Set the desired columns alignment
        - the elements of the array should be either "l", "c" or "r":
            * "l": column flushed left
            * "c": column centered
            * "r": column flushed right
        """

        self._check_row_size(array)
        self._align = array
        return self

    def set_cols_valign(self, array):
        """Set the desired columns vertical alignment
        - the elements of the array should be either "t", "m" or "b":
            * "t": column aligned on the top of the cell
            * "m": column aligned on the middle of the cell
            * "b": column aligned on the bottom of the cell
        """

        self._check_row_size(array)
        self._valign = array
        return self

    def set_cols_dtype(self, array):
        """Set the desired columns datatype for the cols.
        - the elements of the array should be either a callable or any of
          "a", "t", "f", "e" or "i":
            * "a": automatic (try to use the most appropriate datatype)
            * "t": treat as text
            * "f": treat as float in decimal format
            * "e": treat as float in exponential format
            * "i": treat as int
            * a callable: should return formatted string for any value given
        - by default, automatic datatyping is used for each column
        """

        self._check_row_size(array)
        self._dtype = array
        return self

    def set_cols_width(self, array):
        """Set the desired columns width
        - the elements of the array should be integers, specifying the
          width of each column. For example:
                [10, 20, 5]
        """

        self._check_row_size(array)
        try:
            array = list(map(int, array))
            if reduce(min, array) <= 0:
                raise ValueError
        except ValueError:
            sys.stderr.write("Wrong argument in column width specification\n")
            raise
        self._width = array
        return self

    def set_precision(self, width):
        """Set the desired precision for float/exponential formats
        - width must be an integer >= 0
        - default value is set to 3
        """

        if not type(width) is int or width < 0:
            raise ValueError('width must be an integer greater then 0')
        self._precision = width
        return self

    def header(self, array):
        """Specify the header of the table
        """

        self._check_row_size(array)
        self._header = list(map(obj2unicode, array))
        return self

    def add_row(self, array):
        """Add a row in the rows stack
        - cells can contain newlines and tabs
        """

        self._check_row_size(array)

        if not hasattr(self, "_dtype"):
            self._dtype = ["a"] * self._row_size

        cells = []
        for i, x in enumerate(array):
            cells.append(self._str(i, x))
        self._rows.append(cells)
        return self

    def add_rows(self, rows, header=True):
        """Add several rows in the rows stack
        - The 'rows' argument can be either an iterator returning arrays,
          or a by-dimensional array
        - 'header' specifies if the first row should be used as the header
          of the table
        """

        # nb: don't use 'iter' on by-dimensional arrays, to get a
        #     usable code for python 2.1
        if header:
            if hasattr(rows, '__iter__') and hasattr(rows, 'next'):
                self.header(rows.next())
            else:
                self.header(rows[0])
                rows = rows[1:]
        for row in rows:
            self.add_row(row)
        return self

    def draw(self):
        """Draw the table
        - the table is returned as a whole string
        """

        if not self._header and not self._rows:
            return
        self._compute_cols_width()
        self._check_align()
        out = ""
        if self._has_border():
            out += self._hline()
        if self._header:
            out += self._draw_line(self._header, isheader=True)
            if self._has_header():
                out += self._hline_header()
        length = 0
        for row in self._rows:
            length += 1
            out += self._draw_line(row)
            if self._has_hlines() and length < len(self._rows):
                out += self._hline()
        if self._has_border():
            out += self._hline()
        return out[:-1]

    @classmethod
    def draw_env(cls):
        import os
        return cls.draw_dict(os.environ)

    @classmethod
    def draw_df(cls, df):
        tb = cls()
        tb.header(df.columns)
        tb.add_rows(df.values, header=False)
        return tb.draw()

    @classmethod
    def draw_dict(cls, dic):
        tb = cls()
        tb.add_rows(list(dic.items()), header=False)
        return tb.draw()

    @classmethod
    def _to_float(cls, x):
        if x is None:
            raise FallbackToText()
        try:
            return float(x)
        except (TypeError, ValueError):
            raise FallbackToText()

    @classmethod
    def _fmt_int(cls, x, **kw):
        """Integer formatting class-method.
        """
        if type(x) == int:
            return str(x)
        else:
            return str(int(round(cls._to_float(x))))

    @classmethod
    def _fmt_float(cls, x, **kw):
        """Float formatting class-method.
        - x parameter is ignored. Instead kw-argument f being x float-converted
          will be used.
        - precision will be taken from `n` kw-argument.
        """
        n = kw.get('n')
        return '%.*f' % (n, cls._to_float(x))

    @classmethod
    def _fmt_exp(cls, x, **kw):
        """Exponential formatting class-method.
        - x parameter is ignored. Instead kw-argument f being x float-converted
          will be used.
        - precision will be taken from `n` kw-argument.
        """
        n = kw.get('n')
        return '%.*e' % (n, cls._to_float(x))

    @classmethod
    def _fmt_text(cls, x, **kw):
        """String formatting class-method."""
        return obj2unicode(x)

    @classmethod
    def _fmt_auto(cls, x, **kw):
        """auto formatting class-method."""
        f = cls._to_float(x)
        if abs(f) > 1e8:
            fn = cls._fmt_exp
        elif f != f:  # NaN
            fn = cls._fmt_text
        elif f - round(f) == 0:
            fn = cls._fmt_int
        else:
            fn = cls._fmt_float
        return fn(x, **kw)

    def _str(self, i, x):
        """Handles string formatting of cell data
            i - index of the cell datatype in self._dtype
            x - cell data to format
        """
        FMT = {
            'a': self._fmt_auto,
            'i': self._fmt_int,
            'f': self._fmt_float,
            'e': self._fmt_exp,
            't': self._fmt_text,
        }

        n = self._precision
        dtype = self._dtype[i]
        try:
            if callable(dtype):
                return dtype(x)
            else:
                return FMT[dtype](x, n=n)
        except FallbackToText:
            return self._fmt_text(x)

    def _check_row_size(self, array):
        """Check that the specified array fits the previous rows size
        """

        if not self._row_size:
            self._row_size = len(array)
        elif self._row_size != len(array):
            raise ArraySizeError("array should contain %d elements" \
                                 % self._row_size)

    def _has_vlines(self):
        """Return a boolean, if vlines are required or not
        """

        return self._deco & Besttable.VLINES > 0

    def _has_hlines(self):
        """Return a boolean, if hlines are required or not
        """

        return self._deco & Besttable.HLINES > 0

    def _has_border(self):
        """Return a boolean, if border is required or not
        """

        return self._deco & Besttable.BORDER > 0

    def _has_header(self):
        """Return a boolean, if header line is required or not
        """

        return self._deco & Besttable.HEADER > 0

    def _hline_header(self):
        """Print header's horizontal line
        """

        return self._build_hline(True)

    def _hline(self):
        """Print an horizontal line
        """

        if not self._hline_string:
            self._hline_string = self._build_hline()
        return self._hline_string

    def _build_hline(self, is_header=False):
        """Return a string used to separated rows or separate header from
        rows
        """
        horiz = self._char_horiz
        if (is_header):
            horiz = self._char_header
        # compute cell separator
        s = "%s%s%s" % (horiz, [horiz, self._char_corner][self._has_vlines()],
                        horiz)
        # build the line
        l = s.join([horiz * n for n in self._width])
        # add border if needed
        if self._has_border():
            l = "%s%s%s%s%s\n" % (self._char_corner, horiz, l, horiz,
                                  self._char_corner)
        else:
            l += "\n"
        return l

    def _len_cell(self, cell):
        """Return the width of the cell
        Special characters are taken into account to return the width of the
        cell, such like newlines and tabs
        """

        cell_lines = cell.split('\n')
        maxi = 0
        for line in cell_lines:
            length = 0
            parts = line.split('\t')
            for part, i in zip(parts, list(range(1, len(parts) + 1))):
                length = length + len(part)
                if i < len(parts):
                    length = (length // 8 + 1) * 8
            maxi = max(maxi, length)
        return maxi

    def _compute_cols_width(self):
        """Return an array with the width of each column
        If a specific width has been specified, exit. If the total of the
        columns width exceed the table desired width, another width will be
        computed to fit, and cells will be wrapped.
        """

        if hasattr(self, "_width"):
            return
        maxi = []
        if self._header:
            maxi = [self._len_cell(x) for x in self._header]
        for row in self._rows:
            for cell, i in zip(row, list(range(len(row)))):
                try:
                    maxi[i] = max(maxi[i], self._len_cell(cell))
                except (TypeError, IndexError):
                    maxi.append(self._len_cell(cell))

        ncols = len(maxi)
        content_width = sum(maxi)
        deco_width = 3 * (ncols - 1) + [0, 4][self._has_border()]
        if self._max_width and (content_width + deco_width) > self._max_width:
            """ content too wide to fit the expected max_width
            let's recompute maximum cell width for each cell
            """
            if self._max_width < (ncols + deco_width):
                raise ValueError('max_width too low to render data')
            available_width = self._max_width - deco_width
            newmaxi = [0] * ncols
            i = 0
            while available_width > 0:
                if newmaxi[i] < maxi[i]:
                    newmaxi[i] += 1
                    available_width -= 1
                i = (i + 1) % ncols
            maxi = newmaxi
        self._width = maxi

    def _check_align(self):
        """Check if alignment has been specified, set default one if not
        """

        if not hasattr(self, "_header_align"):
            self._header_align = ["c"] * self._row_size
        if not hasattr(self, "_align"):
            self._align = ["l"] * self._row_size
        if not hasattr(self, "_valign"):
            self._valign = ["t"] * self._row_size

    def _draw_line(self, line, isheader=False):
        """Draw a line
        Loop over a single cell length, over all the cells
        """

        line = self._splitit(line, isheader)
        space = " "
        out = ""
        for i in range(len(line[0])):
            if self._has_border():
                out += "%s " % self._char_vert
            length = 0
            for cell, width, align in zip(line, self._width, self._align):
                length += 1
                cell_line = cell[i]
                fill = width - len(cell_line)
                if isheader:
                    align = self._header_align[length - 1]
                if align == "r":
                    out += fill * space + cell_line
                elif align == "c":
                    out += (int(fill / 2) * space + cell_line \
                            + int(fill / 2 + fill % 2) * space)
                else:
                    out += cell_line + fill * space
                if length < len(line):
                    out += " %s " % [space, self._char_vert][self._has_vlines()]
            out += "%s\n" % ['', space + self._char_vert][self._has_border()]
        return out

    def _splitit(self, line, isheader):
        """Split each element of line to fit the column width
        Each element is turned into a list, result of the wrapping of the
        string to the desired width
        """

        line_wrapped = []
        for cell, width in zip(line, self._width):
            array = []
            for c in cell.split('\n'):
                if c.strip() == "":
                    array.append("")
                else:
                    array.extend(textwrapper(c, width))
            line_wrapped.append(array)
        max_cell_lines = reduce(max, list(map(len, line_wrapped)))
        for cell, valign in zip(line_wrapped, self._valign):
            if isheader:
                valign = "t"
            if valign == "m":
                missing = max_cell_lines - len(cell)
                cell[:0] = [""] * int(missing / 2)
                cell.extend([""] * int(missing / 2 + missing % 2))
            elif valign == "b":
                cell[:0] = [""] * (max_cell_lines - len(cell))
            else:
                cell.extend([""] * (max_cell_lines - len(cell)))
        return line_wrapped


if __name__ == '__main__':
    table = Besttable()
    table.set_cols_align(["l", "r", "c"])
    table.set_cols_valign(["t", "m", "b"])
    table.add_rows([["Name", "Age", "Nickname"],
                    ["Mr\nXavier\nHuon", 32, "Xav'"],
                    ["Mr\nBaptiste\nClement", 1, "Baby"],
                    ["Mme\nLouise\nBourgeau", 28, "Lou\n \nLoue"]])
    print(table.draw() + "\n")

    table = Besttable()
    table.set_deco(Besttable.HEADER)
    table.set_cols_dtype(['t',  # text
                          'f',  # float (decimal)
                          'e',  # float (exponent)
                          'i',  # integer
                          'a'])  # automatic
    table.set_cols_align(["l", "r", "r", "r", "l"])
    table.add_rows([["text", "float", "exp", "int", "auto"],
                    ["abcd", "67", 654, 89, 128.001],
                    ["efghijk", 67.5434, .654, 89.6, 12800000000000000000000.00023],
                    ["lmn", 5e-78, 5e-78, 89.4, .000000000000128],
                    ["opqrstu", .023, 5e+78, 92., 12800000000000000000000]])
    print(table.draw())

    print(table.draw_env())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kling
# @Time         : 2024/11/28 16:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 注册任务

from meutils.async_utils import async_to_sync_pro
from meutils.async_task import worker, shared_task

from meutils.apis.kling import kolors_virtual_try_on


@shared_task(pydantic=True, retry_kwargs={'max_retries': 5, 'countdown': 10})
@async_to_sync_pro
async def create_task(request: kolors_virtual_try_on.TryOnRequest, **kwargs):
    if isinstance(request, dict):
        request = kolors_virtual_try_on.TryOnRequest(**request)

    response = await kolors_virtual_try_on.create_task(request)
    return response.model_dump()


if __name__ == '__main__':
    pass
    # create_task.apply_async()
    create_task.apply_async(kwargs={"request": kolors_virtual_try_on.TryOnRequest().model_dump()})



# -*- coding: utf-8 -*-
"""
Created on Mon Aug 21 09:58:28 2023

@author: zlf
"""

import os
import email
import imaplib
import time
from email.utils import parseaddr,parsedate
from email.header import decode_header


# 自动生成文件名
def auto_file_name(file_name, local_path):
    try:
        # 分割文件名，反回其文件名和扩展名组成的元组
        name_suffix = os.path.splitext(file_name)
        num = 1
        while True:
            # 重新拼接文件名
            filename = f"{local_path}/{name_suffix[0]}({num}){name_suffix[-1]}"  # file(1).txt
            # 判断本地是否存在该文件
            isFile = os.path.isfile(filename)
            if isFile:
                num += 1
            else:
                break
        return filename
    except Exception as e:
        raise Exception(f"自动生成文件名发生异常抛出，原因：{e}")

# 字符编码转换
def decode_str(str_in):
    value, charset = decode_header(str_in)[0]
    if charset:
        value = value.decode(charset)
    return value

# def get_att(msg, savePath, subject):
#     for part in msg.walk():
#         if part.get_content_maintype() == 'multipart':
#             continue
#         # if part.get_content_maintype() == 'application':
#         #     continue
#         if part.get('Content-Disposition') is None:
#             continue
#         fileName = part.get_filename()
#         # 如果文件名为纯数字、字母时不需要解码，否则需要解码
#
#         # try:
#         #     fileName = decode_header(fileName)[0][0].decode(decode_header(fileName)[0][1])
#         # except Exception as e:
#         #     print(fileName, str(e), sep='>>>')
#
#
#         if fileName:
#             # emailPath = os.path.join(savePath, subject)
#             # if not os.path.exists(emailPath):
#             #     os.makedirs(emailPath)
#             filePath = os.path.join(savePath, fileName)
#             try:
#                 fp = open(filePath, 'wb')
#                 fp.write(part.get_payload(decode=True))
#                 fp.close()
#             except Exception as e:
#                 print(subject, str(e), sep='>>>')
#                 pass
#     return

def get_att(msg,savePath,subject):
    print("正在读取:"+subject)
    for part in msg.walk():
        fileName = part.get_filename()
        h = email.header.Header(fileName)
        dh = email.header.decode_header(h)
        if dh:
            filename = dh[0][0]
            filename = decode_str(str(filename, dh[0][1]))
            # print(filename)
            if os.path.splitext(filename)[-1] == '.xlsx' or os.path.splitext(filename)[-1] == '.xls':
                filePath = os.path.join(savePath, filename)
                # if not os.path.exists(filePath):
                data = part.get_payload(decode=True)
                fp = open(filePath, 'wb')
                fp.write(data)
                fp.close()
    return

def get_email(json_dict):
    try:
        emailType = json_dict["emailType"]  # 邮箱类型
        emailAddress = json_dict["emailAddress"]  # 邮箱账号
        emailPassword = json_dict["emailPassword"]  # 用户密码，授权码
        emailCount = json_dict["emailCount"]  # 邮件数量
        isReadEmail = json_dict["isReadEmail"]  # 是否仅未读邮件
        isSaveAttachment = json_dict["isSaveAttachment"]  # 是否保存附件
        savePath = json_dict["savePath"]
        startTime = json_dict['startTime']
        endTime = json_dict['endTime']
        smtpData = {
            "0": "imap.qq.com",  # qq邮箱
            "1": "imap.126.com",  # 126 邮箱
            "2": "imap.163.com",  # 163 邮箱
            "3": "mail.nesc.cn"
        }
        if emailType in ["0", "1", "2","3"]:
            mailHost = smtpData[emailType]
        else:
            raise Exception("暂时不支持该邮箱服务器，请重新选择新的邮箱服务器")

        if emailType == "2" or emailType == "1":
            imaplib.Commands['ID'] = 'AUTH'
            server = imaplib.IMAP4_SSL(mailHost)
            server.login(emailAddress, emailPassword)
            # 此处用于规避163和126获取邮件时会报错
            args = ("name", emailAddress, "contact", emailAddress, "version", "1.0.0", "vendor", "myclient")
            typ, dat = server._simple_command('ID', '("' + '" "'.join(args) + '")')
        else:
            # 连接pop服务器。如果没有使用SSL，将IMAP4_SSL()改成IMAP4()即可其他都不需要做改动
            server = imaplib.IMAP4(mailHost)
            # 登录--发送者账号和口令
            server.login(emailAddress, emailPassword)

        # 邮箱中的文件夹，默认为'INBOX'
        inbox = server.select("INBOX")
        # 是否仅未读邮件
        if isReadEmail:
            # 搜索匹配的邮件，第一个参数是字符集，None默认就是ASCII编码，第二个参数是查询条件，这里的ALL就是查找全部 UnSeen:未读邮件
            type1, emailData = server.search(None, "UnSeen")
        else:
            type1, emailData = server.search(None, "All")
        # 邮件列表,使用空格分割得到邮件索引
        msgList = emailData[0].split()
        msgList.reverse()

        for msg in msgList:
            type1, datas = server.fetch(msg, '(RFC822)')
            message = email.message_from_bytes(datas[0][1])
            # 获取标题
            subject = decode_str(message.get('subject'))
            mail_dt = message.get('date')
            date = time.strftime("%Y-%m-%d %H:%M:%S", parsedate(mail_dt))
            if date>=startTime and date<=endTime:
                if isSaveAttachment:
                    if savePath is None or savePath == "":
                        #如果没给保存路径，给与一个默认路径,不进行报错提示
                        savePath='../data'
                        # raise Exception("附件保存目录为空，请检查附件保存目录是否输入正确")
                    try:
                        get_att(message, savePath, subject)
                    except Exception as e:
                        print(str(e))
            elif date>endTime:
                continue
            elif date<startTime:
                # 倒序读邮件，如果时间下于开始时间，那么后面的邮件都不需要读了，直接终止程序
                break
        # 关闭连接
        server.close()
        print('嘿！！  下载好了呦！！<(￣︶￣)↗[GO!]')
    except Exception as e:
        print(str(e))
        # raise Exception(f'接收邮件异常抛出，原因: {e}')
    return
if __name__ == '__main__':
    json_dict={
        'emailType':'3',
        'emailAddress': '邮箱账号',
        'emailPassword': '密码',
        'emailCount': '',     # 邮件数量
        'isReadEmail': False, # 是否仅未读邮件
        'isSaveAttachment':True,# 是否保存附件
        'savePath': '../data',  # 保存目录
        'startTime':'2023-08-31 00:00:00', # 时间范围
        'endTime': '2023-09-07 15:00:00', # 时间范围
    }
    get_email(json_dict)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2024/10/28 20:57
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

import jwt
import time
import datetime
#
# # Header and payload
# header = {
#     "alg": "HS512",
#     "type": "JWT"
# }
#
#
# payload = {
#     "jti": "80004477",
#     "rol": "ROLE_REGISTER",
#     "iss": "OpenXLab",
#     "clientId": "lkzdx57nvy22jkpq9x2w",
#     "phone": "",
#     "uuid": "73a8d9b0-8bbf-4973-9b71-4b687ea23a78",
#     "email": "313303303@qq.com",
#
#     "iat": int(time.time()),
#     "exp": int(time.time()) + 3600
# }
#
# # Your secret key
# secret = ""
#
# # Create the JWT
# token = jwt.encode(payload, secret, algorithm="HS512", headers=header)
#
# print(token)



@lru_cache()
def decode_jwt_token(jwt_token):
    # 提取 payload 部分
    payload_part = jwt_token.split('.')[1]

    # 对 payload 进行 Base64 解码
    try:
        # 尝试标准 Base64 解码
        decoded_payload = base64.b64decode(payload_part + '==', validate=True).decode('utf-8')
    except base64.binascii.Error:
        # 尝试 URL 安全的 Base64 解码
        decoded_payload = base64.urlsafe_b64decode(payload_part + '==').decode('utf-8')

    # 解析 JSON 字符串
    payload = json.loads(decoded_payload)

    return payload


if __name__ == '__main__':
    def get_base_url(token):
        if "小螺帽" not in str(decode_jwt_token(token)):
            return "BASE_URL_ABROAD"
        else:
            return "BASE_URL"

    jwt_token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NDg3Mzg4MTQsInVzZXIiOnsiaWQiOiIyMjkwODQ3NTA2MDEzODgwMzciLCJuYW1lIjoi5bCP6J665bi9ODAzNyIsImF2YXRhciI6Imh0dHBzOi8vY2RuLmhhaWx1b2FpLmNvbS9wcm9kL3VzZXJfYXZhdGFyLzE3MDYyNjc3MTEyODI3NzA4NzItMTczMTk0NTcwNjY4OTY1ODk2b3ZlcnNpemUucG5nIiwiZGV2aWNlSUQiOiIyNDM3MTMyNTI1NDU5ODY1NjIiLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.o0SoZMSTWkXNHxJjt3Ggby5MJWSfd-rnK_I95T_WMP8"

    # jwt_token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NTAxMjg4NTIsInVzZXIiOnsiaWQiOiIzNzQwMTM3NzUyNzg4ODY5MTciLCJuYW1lIjoiTmFodWVsIE1vbGluYSIsImF2YXRhciI6IiIsImRldmljZUlEIjoiMzEzMzc0MTIyMjEyMjc4MjczIiwiaXNBbm9ueW1vdXMiOmZhbHNlfX0.uxTtDTcPT07piVA-x3N2ms2VrRN3JwcU99g_HJLwqLE"
    print(get_base_url(jwt_token))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openai_files.py
# @Time         : 2025/2/25 12:16
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from openai import OpenAI, AsyncOpenAI

client = AsyncOpenAI(
    base_url="https://all.chatfire.cn/qwen/v1",
    api_key="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6IjMxMGNiZGFmLTM3NTQtNDYxYy1hM2ZmLTllYzgwMDUzMjljOSIsImV4cCI6MTc0MzAzNTk4OH0.GVAoSFtK94a9CgxqHCEnxzAnRi7gafIvYyH9mIJUh4s"
)


async def upload():
    file_object = await client.files.create(
        file=Path("招标文件备案表（第二次）.pdf"),
        purpose="file-extract"
    )
    return file_object


if __name__ == '__main__':
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : delilegal
# @Time         : 2024/9/20 17:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.pandas_utils import to_html_plus

from meutils.notice.feishu import send_message as _send_message
from meutils.db.redis_db import redis_client, redis_aclient
from meutils.config_utils.lark_utils import aget_spreadsheet_values, get_next_token_for_polling

from meutils.llm.utils import oneturn2multiturn
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, ChatCompletionRequest, CompletionUsage
from meutils.schemas.oneapi import REDIRECT_MODEL

send_message = partial(
    _send_message,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e0db85db-0daf-4250-9131-a98d19b909a9",
    title=__name__
)

BASE_URL = "https://www.delilegal.com"

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=DtiUmU"


async def check_token(token: Optional[str] = None):
    headers = {
        "authorization": token
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        response = await client.get("/ai/api/v1/gptIm/getLimitUser")

        logger.debug(response.status_code)
        logger.debug(response.text)

        if response.is_success:
            return response.json()['body']  #####


@alru_cache(ttl=3600)
@retrying(predicate=lambda r: r is None)
async def create_session(token: Optional[str] = None):
    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL)
    headers = {
        "authorization": token
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        response = await client.post("/ai/api/v1/gptIm/createSession")

        logger.debug(response.status_code)
        logger.debug(response.text)

        if response.is_success:
            return response.json()["body"]["sessionId"]


async def create(request: ChatCompletionRequest, token: Optional[str] = None, answer_again: bool = False):
    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL)

    sessionId = await create_session(token)

    headers = {
        "authorization": token,
    }
    payload = {
        "sessionId": sessionId,
        "question": request.last_content,
        # "question": oneturn2multiturn(request.messages),

        "answerAgain": answer_again,
        "qaId": "",
        "tabType": "lawQa" if 'viewpoint' not in request.model else "viewpointQa"
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        async with client.stream("POST", "/ai/api/v1/gptIm/sendGptMessageStreamV3", json=payload) as response:
            yield "\n"  # 提升首字速度

            logger.debug(response.status_code)
            # logger.debug(response.text)

            _chunk = ""
            async for chunk in response.aiter_lines():
                if (chunk := chunk.strip("data:")) and chunk.startswith("{"):
                    # logger.debug(chunk)
                    try:
                        chunk = json.loads(chunk)
                        data = chunk.get("data", {})
                        chunk = data.get("lawQaText", "") or data.get("viewpointQaText", "")

                        for i in ["lawQaRelatedLaws", "lawQaRelatedCases", "viewpointQaRelatedArticles"]:
                            if data.get(i):
                                chunk = ""  # 最好不返回
                                df = pd.DataFrame(data[i])
                                filename = to_html_plus(
                                    df,
                                    title=i,
                                    subtitle="ChatfireLaw",
                                    return_filename=True
                                )
                                yield f"\n\n[{i}](https://api.chatfire.cn/render/{filename})"

                        yield chunk.replace(_chunk, "")
                        _chunk = chunk
                    except Exception as e:
                        _ = f"{e}\n{chunk}"
                        logger.error(_)
                        send_message(_)
                        yield ""
                # else:
                #     logger.debug(chunk)


if __name__ == '__main__':
    token = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJzdWIiOiJmbm9wc3J6eHFvY3V2Y2ZtY3BwZGNpcW54eWFzY25sYSIsIm5iZiI6MTcyNjgyMzgyNCwiZXhwIjoxNzI3NDI4NjI0LCJpYXQiOjE3MjY4MjM4MjQsImp0aSI6IlppbERZaElKZXF4RFRncnMifQ.tbx8XW9sEOz1bY-c0JrKMJBd3mAoTBRXhvGIotmPiR4"

    # arun(create_session(token))
    # arun(check_token(token))

    model = "viewpointQaText"
    model = ""

    request = ChatCompletionRequest(model=model, messages=[
        {'role': 'user',
         'content': '客人吃出苍蝇超出十倍的赔偿怎么办?\n\n先给出准确答案➕原因理由➕再法律依据➕再相关延伸'}
    ])

    arun(create(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : google2openai
# @Time         : 2025/4/1 13:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 
"""
1. 生图 编辑图 多轮
2. 文件上传、大文件问答

注意 ：借助 File API，您最多可为每个项目存储 20 GB 的文件，每个文件的大小上限为 2 GB。文件会存储 48 小时。您可以在该时间段内使用 API 密钥访问这些数据，但无法从 API 下载这些数据。在已推出 Gemini API 的所有地区，此功能均可免费使用。

"""
from meutils.pipe import *
from meutils.schemas.openai_types import CompletionRequest

from google import genai
from google.genai.types import HttpOptions, GenerateContentConfig, Content, HarmCategory, HarmBlockThreshold, Part

# Content(role="user", parts=[Part.from_text(text=prompt)]),
# Content(role="model", parts=[Part.from_text(text="Ok")]),

config = GenerateContentConfig(

    temperature=0.7,
    top_p=0.8,
    # response_modalities=['Text', 'Image'],

    # 公民诚信类别的默认屏蔽阈值为 Block none（对于别名为 gemini-2.0-flash、gemini-2.0-pro-exp-02-05 和 gemini-2.0-flash-lite 的 gemini-2.0-flash-001），适用于 Google AI Studio 和 Gemini API；仅适用于 Google AI Studio 中的所有其他模型的 Block most。
    # safety_settings=[
    #     SafetySetting(
    #         category=HarmCategory.HARM_CATEGORY_HATE_SPEECH,
    #         threshold=HarmBlockThreshold.BLOCK_LOW_AND_ABOVE,
    #     ),
    # ]
)


# self._http_options.base_url = 'https://generativelanguage.googleapis.com/'
# self._http_options.api_version = 'v1beta'
client = genai.Client(
    api_key="AIzaSyAlpq4kR9ZP0NwaqQzqHtDKqiV8PLdUqnA",
    http_options=HttpOptions(
        base_url="https://all.chatfire.cc/genai"
    )
)

file = "/Users/betterme/PycharmProjects/AI/QR.png"
#
# file_object = client.files.upload(file=file)
# prompt = "一句话总结"

# file_object = client.aio.files.upload(file=file)
# https://generativelanguage.googleapis.com/v1beta/files/ickgffcfb9zl
#
# contents = ('Hi, can you create a 3d rendered image of a pig '
#             'with wings and a top hat flying over a happy '
#             'futuristic scifi city with lots of greenery?')
#
# prompt = "9.11 9.8哪个大呢"

uri='https://generativelanguage.googleapis.com/v1beta/files/88n7hk8tau7g'

# client.aio.chats.create(
#     model="gemini-2.0-flash-exp-image-generation",
# )

response = client.models.generate_content(
    model="gemini-2.0-flash",
    contents=[
        '解释下',
        Part.from_uri(file_uri=uri, mime_type='image/png')
    ],

    # model="gemini-2.5-pro-exp-03-25",
    # model="gemini-2.0-flash",

    # contents=[
    #     Part.from_uri(file_uri='https://generativelanguage.googleapis.com/v1beta/files/test', mime_type='image/png'),
    #
    #           "一句话总结"],
    config=config
)

# client.aio.
# client.aio.chats.create()

if __name__ == '__main__':
    # arun(file_object)
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : sys_monitor
# @Time         : 2022/9/23 下午1:44
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
psutil.cpu_count()
psutil.cpu_percent()
psutil.cpu_stats()
psutil.disk_io_counters()

psutil.process_iter()

psutil.swap_memory
psutil.virtual_memory()

"""

import psutil


psutil.test()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : mystic
# @Time         : 2024/8/21 13:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.config_utils.lark_utils import get_spreadsheet_values, get_next_token_for_polling
from meutils.schemas.openai_types import ImageRequest, ImagesResponse
from meutils.apis.translator import deeplx
from meutils.schemas.translator_types import DeeplxRequest
from meutils.schemas.image_types import ASPECT_RATIOS

from meutils.decorators.retry import retrying

from meutils.notice.feishu import send_message as _send_message

# https://www.mystic.ai/v4/runs/run_64c682fd3a6f4da4bbdacec7720add5f

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=7VYdjp"

BASE_URL = "https://www.mystic.ai/v4/runs"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


@retrying(max_retries=5, title=__name__)
async def create_image(request: ImageRequest, token: Optional[str] = None, async_run: bool = False):
    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL)

    prompt = (await deeplx.translate(DeeplxRequest(text=request.prompt, target_lang="EN"))).get("data")

    width, height = map(int, ASPECT_RATIOS.get(request.size, '1024x1024').split('x'))

    if 'pro' in request.model:
        width = max(width, 1440)
        height = max(height, 1440)

        payload = {
            "pipeline": "pipeline_0710417cf4464e72b03dc9ef17773253",
            "inputs": [
                {
                    "type": "dictionary",
                    "value": {
                        "prompt": prompt,
                        "prompt_upsampling": False,

                        "width": width,
                        "height": height,

                        "steps": 15,
                        "guidance": 3,

                        "variant": "flux.1-pro",

                        "safety_tolerance": request.nsfw_level,  # 小等于6
                        "interval": 2,
                        "seed": 0,

                    }
                }
            ],
            "async_run": async_run
        }
    else:  # dev
        payload = {
            "pipeline": "pipeline_8a895066ef054c2e89cdf316611a1937",
            "inputs": [
                {
                    "type": "string",
                    "value": prompt
                },
                {
                    "type": "dictionary",
                    "value": {

                        "height": 1024,
                        "width": 1024,

                        "guidance_scale": 4,
                        "max_sequence_length": 256,
                        "num_images_per_prompt": 1,
                        "num_inference_steps": 15,
                        "seed": 1
                    }
                }
            ],
            "async_run": True
        }

    headers = {
        'Cookie': token,
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        response = await client.post(BASE_URL, json=payload)

        logger.debug(response.status_code)
        logger.debug(response.text)

        if response.is_success:
            data = response.json()
            logger.debug(data)

            return ImagesResponse(data=[data['outputs'][0]['file']])


if __name__ == '__main__':
    arun(create_image(ImageRequest(prompt="一条狗")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : web_search
# @Time         : 2025/3/18 20:15
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.caches import rcache

from meutils.async_utils import sync_to_async

from meutils.llm.clients import zhipuai_sdk_client
from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, CompletionRequest, ChatCompletionChunk


def convert_citations(text):
    """
    # 示例使用
    text = "这是一段包含【1†source】和【2†source】的文本"
    result = convert_citations(text)
    print(result)  # 输出: 这是一段包含[^1]和[^2]的文本
    :param text:
    :return:
    """
    # 匹配【数字†source】格式的引用
    pattern = r'【(\d+)†source】'

    # 替换为[^数字]格式
    converted = re.sub(pattern, r'[^\1]', text)

    return converted


class Completions(object):

    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key

    # @rcache(noself=True, ttl=15 * 60)
    @sync_to_async
    def query(self, q: str):
        data = list(self.create(q))
        return {"data": data}

    def create(self, request: Union[CompletionRequest, str]):

        q = request.last_user_content if isinstance(request, CompletionRequest) else request
        q = f"{q} 【最新动态、相关信息或新闻】"

        chunks = zhipuai_sdk_client.assistant.conversation(

            assistant_id="659e54b1b8006379b4b2abd6",  # 搜索智能体
            conversation_id=None,
            model="glm-4-assistant",  # assistant-ppt
            messages=[
                {
                    "role": "user",
                    "content": [{
                        "type": "text",
                        # "text": "北京未来七天气温，做个折线图",
                        # "text": "画条狗"
                        "text": q,

                    }]
                }
            ],
            stream=True,
            attachments=None,
            metadata=None
        )

        references = []
        for chunk in chunks:
            # logger.debug(chunk)

            delta = chunk.choices[0].delta
            if hasattr(delta, "tool_calls") and delta.tool_calls:
                tool_call = delta.tool_calls[0].model_dump()
                # logger.debug(tool_call)
                tool_type = tool_call.get("type", "")  # web_browser
                references += tool_call.get(tool_type, {}).get("outputs") or []  # title link content

                # logger.debug(f"references: {references}")
                continue

            if isinstance(request, CompletionRequest):
                if references:
                    urls = [f"[^{i}]: [{ref['title']}]({ref['link']})\n" for i, ref in enumerate(references, 1)]

                    for url in urls:
                        yield url

                    references = []

                delta = chat_completion_chunk.choices[0].delta.model_construct(**delta.model_dump())
                chat_completion_chunk.choices[0].delta = delta
                yield chat_completion_chunk

            else:
                yield references
                break


if __name__ == '__main__':
    request = CompletionRequest(
        # model="baichuan4-turbo",
        # model="xx",
        # model="deepseek-r1",
        # model="deepseek-r1:1.5b",
        model="model",

        # model="moonshot-v1-8k",
        # model="doubao",

        messages=[
            {"role": "user", "content": "周杰伦"}
        ],

        stream=True
    )
    print(Completions().create(request))
    # arun(Completions().query(request.last_user_content))
